import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.nio.*;
import java.util.ArrayList;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;

public class Index {
    DocSearch ds;
    boolean debug=true;
    WebPageMetaData wpmd;
    final static String wordFileType="Cp1252";
    final static String META_TAG="<meta";
    final static String BODY_TAG="<body";
    final static String BODY_TAG_END="</body";
    final static String TITLE_TAG="<title";
    final static String TITLE_TAG_END="</title";
    final static String SCRIPT_TAG="<script";
    final static String SCRIPT_TAG_END="</script";
    StringBuffer notesBuf=new StringBuffer();
    StringBuffer newItsBuf=new StringBuffer();
    StringBuffer modItsItsBuf=new StringBuffer();
    StringBuffer delItsItsBuf=new StringBuffer();
    int totalChanges=0;
    boolean isTextEmailFormat=true;
    boolean doEmail=false;
    int insertMode=0; // 0 = new, 1 = modified, 2 = deleted

public Index(DocSearch ds) {
        this.ds = ds;
}


public int addDocToIndex(String currentFi, IndexWriter writer, 
		DocSearcherIndex di, boolean isCdRomIndx, 
		SpiderUrl spy)    {
	boolean isSpiderFile=di.isSpider;
        int returnInt = 0; 
	/* 0  =  OK, 1 =  failed, 2 = meta robots = noindex.... */
	System.out.println("Adding "+currentFi+" to index...");
        String metaRobot = "";
        boolean thisErr = false;
        synchronized (this)  {
            try  {
                //
                Document doc; 
		OoToText op;
                String urlStr = "";
                String lowerType = "";
                String author = "";
                String keyWords = "";
                String curType = "";
                String curTitle = "";
                String dateStr;
                String curSummary = "";
                File curFile = new File(currentFi);
                long curFileSize = curFile.length();
                int typeInt = 0;
                // get an iterator to iterate over our files
                InputStream is; // for our file
                curType = getFileType(currentFi);
                lowerType = curType.toLowerCase();
                typeInt = Utils.getTypeInt(lowerType);
		if (isSpiderFile) dateStr = DateTimeUtils.getTimeStringForIndex(spy.getLastModified());
		else dateStr = DateTimeUtils.getTimeStringForIndex(curFile.lastModified());
		if (isSpiderFile) urlStr=spy.getUrl();
                else if (di.isWeb) urlStr = Utils.getURL(currentFi, di.replace, di.match);
		else if (isCdRomIndx) urlStr = Utils.getURL(currentFi, di.replace, di.match);
			doc = new Document();
			switch (typeInt) {
                            case 0: // html
			    	    wpmd=getWebPageMetaData(currentFi);
		                    curTitle = wpmd.getTitle();
		                    curSummary = wpmd.getDesc();
			            author = wpmd.getAuthor();
		                    is = new FileInputStream(ds.htmlTextFile);
                                break;
                            case 1: // text
		                    curTitle = getDocTitle(currentFi, typeInt);
		                    curSummary = getDocSummary(currentFi, typeInt);
		                    is = new FileInputStream(currentFi);
                                break;
                            case 2: // ms word
		                    WordProps wp = new WordProps(currentFi, ds.wordTextFile);
            		        wp.getProps();
		                    curTitle = wp.title;
		                    keyWords = wp.keyWords;
            		        author = wp.author;
		                    curSummary = getDocSummary(ds.wordTextFile, typeInt);
					  is = new FileInputStream(ds.wordTextFile);
                                break;
                            case 3: // ms excell
            		        ExcelProps ep = new ExcelProps(currentFi, ds.excelTextFile);
		                    ep.getProps();
            		        curTitle = ep.title;
		                    keyWords = ep.keyWords;
            		        author = ep.author;
		                    curSummary = getDocSummary(ds.excelTextFile, typeInt);
   					  is = new FileInputStream(ds.excelTextFile);
                                break;
                            case 4: // pdf
			        //if (debug) System.out.println("PDF Examination: "+currentFi);
            		      PdfToText pp = new PdfToText(currentFi, ds.pdfTextFile);
				  pp.reset();
				/* 
		                  pp.parse();
				if (pp.success()) {
            		      		curSummary = pp.getSummary();
					if (curSummary.equals("")) curSummary = getDocSummary(ds.pdfTextFile, typeInt);
		                	author = pp.getAuthor();
					keyWords =  pp.getKeywords();
					curTitle = pp.getTitle();
					is = new FileInputStream(ds.pdfTextFile);
					}
				else */
				if (pp.doMultiValent()) { // (pp.foundText()) {
					curSummary = getDocSummary(ds.pdfTextFile, typeInt);
					is = new FileInputStream(ds.pdfTextFile);
					} // end for foundText 
				else if  (pp.didParse()) {
            		      		curSummary = pp.getSummary();
					if (curSummary.equals("")) curSummary = getDocSummary(ds.pdfTextFile, typeInt);
		                	author = pp.getAuthor();
					keyWords =  pp.getKeywords();
					curTitle = pp.getTitle();
					is = new FileInputStream(ds.pdfTextFile);
					}
				else throw new Exception("PDF Parsing Error:Unable to parse document --> "+currentFi);
                                break;
                            case 5: // rtf
		                    RtfToText rp = new RtfToText(currentFi, ds.rtfTextFile);
            		        rp.parse();
		                    curSummary = getDocSummary(ds.rtfTextFile, typeInt);
					  is = new FileInputStream(ds.rtfTextFile);
                                break;
                            case 6: // open office writer
            		        op = new OoToText(currentFi, ds.ooTextFile, ds.ooMetaTextFile);
		                    op.parse();
            		        author = getTagText("creator", ds.ooMetaTextFile);
				keyWords = Utils.nonTagText(getTagText("keywords", ds.ooMetaTextFile));
		                removeAllTags(ds.ooTextFile, ds.ooTextOnlyFile);
            		        curSummary = getDocSummary(ds.ooTextOnlyFile, typeInt);
			        curTitle = getTagText("title", ds.ooMetaTextFile);
		                if (curTitle.trim().equals("")) curTitle = getDocTitle(ds.ooTextOnlyFile, typeInt);
				is = new FileInputStream(ds.ooTextOnlyFile);
                                break;
                            case 7: // open office impress
            		        op = new OoToText(currentFi, ds.ooTextFile, ds.ooMetaTextFile);
		                    op.parse();
            		        author = getTagText("creator", ds.ooMetaTextFile);
				keyWords = Utils.nonTagText(getTagText("keywords", ds.ooMetaTextFile));
		                    removeAllTags(ds.ooTextFile, ds.ooTextOnlyFile);
            		        curSummary = getDocSummary(ds.ooTextOnlyFile, typeInt);
				curTitle = getTagText("title", ds.ooMetaTextFile);
		                if (curTitle.trim().equals("")) curTitle = getDocTitle(ds.ooTextOnlyFile, typeInt);
				is = new FileInputStream(ds.ooTextOnlyFile);
                                break;
                            case 8: // open office calc
            		        op = new OoToText(currentFi, ds.ooTextFile, ds.ooMetaTextFile);
		                    op.parse();
            		        author = getTagText("creator", ds.ooMetaTextFile);
				keyWords = Utils.nonTagText(getTagText("keywords", ds.ooMetaTextFile));
		                    removeAllTags(ds.ooTextFile, ds.ooTextOnlyFile);
            		        curSummary = getDocSummary(ds.ooTextOnlyFile, typeInt);
		        	curTitle = getTagText("title", ds.ooMetaTextFile);
		                if (curTitle.trim().equals("")) curTitle = getDocTitle(ds.ooTextOnlyFile, typeInt);
				is = new FileInputStream(ds.ooTextOnlyFile);
                                break;
                            case 9: // open office draw
            		        op = new OoToText(currentFi, ds.ooTextFile, ds.ooMetaTextFile);
		                    op.parse();
            		        author = getTagText("creator", ds.ooMetaTextFile);
				keyWords = Utils.nonTagText(getTagText("keywords", ds.ooMetaTextFile));
		                    removeAllTags(ds.ooTextFile, ds.ooTextOnlyFile);
            		        curSummary = getDocSummary(ds.ooTextOnlyFile, typeInt);
				curTitle = getTagText("title", ds.ooMetaTextFile);
		                if (curTitle.trim().equals("")) curTitle = getDocTitle(ds.ooTextOnlyFile, typeInt);
				is = new FileInputStream(ds.ooTextOnlyFile);
                                break;
                            default: // text
		                    curTitle = getDocTitle(currentFi, typeInt);
		                    curSummary = getDocSummary(currentFi, typeInt);
		                    is = new FileInputStream(currentFi);
                                break;
				} // end for switching document type
		/* spider url stuff */
		if (isSpiderFile) {
			if (curTitle!=null) {
			
				}
			else curTitle="";
			if (curTitle.trim().equals("")) curTitle=Utils.getNameOnly(urlStr);
			}
		/* repair missing meta data - if needed */
                if (curTitle != null) {  }
                else  curTitle = "";
                if (author != null)  { }
                else   author = "";
                if (keyWords != null) { }
                else  keyWords = "";
                if (curSummary != null) { }
                else   curSummary = "";
                if (curTitle.trim().equals(""))  curTitle = Utils.getNameOnly(currentFi);
                if ((typeInt == 1) && (curSummary.toLowerCase().indexOf("noindex") != -1))   {
                    // if its a text document with NOINDEX in the start of the text - don't index it
                    ds.setStatus("Document " + currentFi  + " PREFERS no indexing.");
                    returnInt = 2;
                }
                else if (typeInt == 0) {
                    // web page - check for meta name = robots content = noindex...
                    metaRobot = getMetaTag(currentFi, "robots");
                    if (metaRobot.toLowerCase().indexOf("noindex") != -1) {
                        thisErr = true;
                        ds.setStatus("Document " + currentFi  + " PREFERS no indexing.");
                        returnInt = 2;
                    }
                }
			// LASTLY ADD OUR DOCUMENT
                if (returnInt == 0) {
		    if (isSpiderFile) doc.add(Field.UnIndexed("md5", spy.getMd5()));
                    doc.add(Field.UnIndexed("path", currentFi));
                    doc.add(Field.Keyword("size", "" + curFileSize));
                    doc.add(Field.Text("type", lowerType));
                    doc.add(Field.Text("author", author));
                    doc.add(Field.Text("mod_date", dateStr));
                    doc.add(Field.Text("keywords", keyWords));
                    doc.add(Field.Text("title", curTitle));
                    doc.add(Field.Text("summary", curSummary));
                    if (typeInt!=2) doc.add(Field.Text("body", new BufferedReader (new InputStreamReader(is))));
		    else doc.add(Field.Text("body", new BufferedReader (new InputStreamReader(is, wordFileType))));
		    // OLD WAY: doc.add(Field.Text("body", (Reader)new InputStreamReader(is, wordFileType)));
                    doc.add(Field.Text("URL", urlStr));
                    writer.addDocument(doc);
		    addToSummary(curTitle, author, dateStr, lowerType, curSummary, urlStr, ""+curFileSize);
                }
                else ds.setStatus("DOCUMENT " + currentFi + " WAS NOT ADDED TO INDEX.");
                is.close();
            }
            catch (Exception aE) {
                ds.setStatus("Error indexing " + currentFi + ":" + aE.toString());
		if (debug) aE.printStackTrace();
                returnInt = 1;
            }
            finally {
                return returnInt;
            }
        }
    }
    
public int spiderIndexNum(int lastFound, String fileName, IndexReader ir) {
        int returnInt = -1;
        synchronized (this) {
            if (lastFound == -1)  lastFound = 0;
            try {
                Document doc;
                String compareName = "";
                int numDocs = ir.maxDoc();
                for (int i = lastFound; i < numDocs; i++) {
                    if (!ir.isDeleted(i)) {
                        doc = ir.document(i);
                        if (doc != null) {
                            compareName = doc.get("URL");
                            if (compareName.equals(fileName))  {
                                returnInt = i;
                                break;
                            }
                        }
                    }
                }
                if (returnInt == -1) {
                    for (int i = lastFound; i > 0; i--) {
                        if (!ir.isDeleted(i)) {
                            doc = ir.document(i);
                            if (doc != null) {
                                compareName = doc.get("URL");
                                //System.out.println("Comparing "+compareName+" to "+fileName);
                                if (compareName.equals(fileName)) {
                                    //System.out.println("MATCH FOUND AT "+i);
                                    returnInt = i;
                                    break;
                                }
                            }
                        }
                    }
                }

                if (returnInt == -1) ds.setStatus("File " + fileName + " not found in index!");
            }
            catch (Exception eI) {
                ds.setStatus("Error determining if doc is already in index!");
                eI.printStackTrace();
            }
            finally {
                return returnInt;
            }
        }
    }

public int indexNum(int lastFound, String fileName, IndexReader ir) {
        int returnInt = -1;
        synchronized (this) {
            if (lastFound == -1)  lastFound = 0;
            try {
                Document doc;
                String compareName = "";
                int numDocs = ir.maxDoc();
                for (int i = lastFound; i < numDocs; i++) {
                    if (!ir.isDeleted(i)) {
                        doc = ir.document(i);
                        if (doc != null) {
                            compareName = doc.get("path");
                            if (compareName.equals(fileName))  {
                                returnInt = i;
                                break;
                            }
                        }
                    }
                }
                if (returnInt == -1) {
                    for (int i = lastFound; i > 0; i--) {
                        if (!ir.isDeleted(i)) {
                            doc = ir.document(i);
                            if (doc != null) {
                                compareName = doc.get("path");
                                //System.out.println("Comparing "+compareName+" to "+fileName);
                                if (compareName.equals(fileName)) {
                                    //System.out.println("MATCH FOUND AT "+i);
                                    returnInt = i;
                                    break;
                                }
                            }
                        }
                    }
                }

                if (returnInt == -1) ds.setStatus("File " + fileName + " not found in index!");
            }
            catch (Exception eI) {
                ds.setStatus("Error determining if doc is already in index!");
                eI.printStackTrace();
            }
            finally {
                return returnInt;
            }
        }
    }


    
public void updateIndex(DocSearcherIndex di) {
        notesBuf=new StringBuffer();
	newItsBuf=new StringBuffer();
	modItsItsBuf=new StringBuffer();
	delItsItsBuf=new StringBuffer();
	totalChanges=0;  long curFileSizeBytes=0;
        int errNum = 0;
        StringBuffer noRobotsBuf = new StringBuffer();
        int numNoIndex = 0;
        int numErrors = 0;
        StringBuffer failedBuf = new StringBuffer();
        int addedSuccessFully = 0;
        failedBuf.append("\n");
        synchronized (this) {
	    if ((di.isCdrom())) { }
	    else if (di.isSpider) {
	    	doSpiderUpdate(di);
	    	}
            else if (!di.path.toLowerCase().endsWith(".zip")) { // not a zip archive
                int numUpdates = 0;
                int numRemovals = 0;
                int numNew = 0;
                try {
                    IndexWriter iw;
                    IndexReader ir = IndexReader.open(di.indexerPath);
                    int numDocs = ir.maxDoc();
                    ds.setStatus("there are " + numDocs + " docs in index " + di.desc + "(" + di.path + ")");
		    //
		    addHeader(di.desc);
                    ArrayList allDocsInIndex = new ArrayList(); // indexed files
                    ArrayList allDocsInFolder = new ArrayList(); // current files
                    ArrayList newDocsToAdd = new ArrayList(); // files to be added that are new
                    String curFiModDate;
                    String realFileModDate;
                    File testFi;
                    String curFiName = "";
                    Document doc;
		    setInsertMode(1); // note we are looking for modified files
                    for (int i = 0; i < numDocs; i++) {
                        if (!ir.isDeleted(i)) {
                            doc = ir.document(i);
                            if (doc != null) {
                                curFiName = doc.get("path");
                                curFiModDate = doc.get("mod_date");
                                testFi = new File(curFiName);
                                if (testFi.exists()) {
                                    allDocsInIndex.add(curFiName);
                                    realFileModDate = DateTimeUtils.getTimeStringForIndex(testFi.lastModified());
                                    if (realFileModDate.equals(curFiModDate)) { }
                                    else {
                                        //System.out.println("TEMPORARILY REMOVING: " + curFiName);
                                        numUpdates++;
                                        ir.delete(i); 
                                        ir.close();
                                        // open writer to add document once again
                                        ds.setStatus("REINDEXING: " + curFiName);
                                        iw = new IndexWriter(di.indexerPath, new StandardAnalyzer(), false);
					/* next line should remove too many files open errors */
					//iw.setUseCompoundFile(true);
                                        addedSuccessFully = addDocToIndex(curFiName, iw, di, di.isCdrom(), null);
                                        iw.close();
                                        // reopen
                                        ir = IndexReader.open(di.indexerPath);
                                        switch (addedSuccessFully) {
                                        case 1: // error
                                            errNum++;
                                            if (errNum < 8) {
                                                failedBuf.append("\n");
                                                failedBuf.append(curFiName);
                                            }
                                            ds.setStatus(ds.dsErrIdxgFi+" " + curFiName);
                                            break;
                                        case 2: // meta robots = noindex
                                            numNoIndex++;
                                            if (numNoIndex < 8) {
                                                noRobotsBuf.append("\n");
                                                noRobotsBuf.append(curFiName);
                                            }
                                            ds.setStatus("No Indexing Meta Requirement found in : " + curFiName);
                                            break;
                                        default: // OK
                                            numUpdates++;
                                            ds.setStatus("Indexing " + curFiName  + " complete.");
                                            break;
                                        } // end of switch
                                    }
                                }
                                 // end for file exists
                                else  {
                                    ds.setStatus("DELETING: " + curFiName);
                                    ir.delete(i);
				    addDelNote(doc);
                                    numRemovals++;
                                }
                                 // end for file not found
                            }
                             // end for doc is not null
                        }
                         // end for not deleted
                        //else System.out.println("Document was null or deleted:"+i);
                    }
                     // end for getting gocs
                    // now add the new files
		    setInsertMode(0);
                    ArrayList folderList = new ArrayList();
                    folderList.add(di.path);
                    int lastFound = 0;
                    int startSubNum = Utils.countSLash(di.path);
                    int maxSubNum = startSubNum + di.depth;
                    String curFolderString = "";
                    String curFold;
                    String[] foldersString;
                    int numFolders = 0;
                    File curFolderFile;
                    int curSubNum = 0;
                    int lastItemNo = 0;
                    int curItemNo = 0;
                    int curSize = 1;
                    String[] filesString;
                    int numFiles = 0;
                    String curFi = "";
                    do {
                        // create our folder file
                        curFolderString = (String)folderList.get(curItemNo);
                        curFolderFile = new File(curFolderString);
                        curSubNum = Utils.countSLash(curFolderString);
                        // handle any subfolders --> add them to our folderlist
                        foldersString = curFolderFile.list(DocSearch.ff);
                        numFolders = foldersString.length;
                        for (int i = 0; i < numFolders; i++) {
                            // add them to our folderlist
                            curFold = curFolderString + DocSearch.pathSep + foldersString[i] + DocSearch.pathSep;
                            curFold = Utils.replaceAll(DocSearch.pathSep + DocSearch.pathSep, curFold, DocSearch.pathSep);
                            folderList.add(curFold);
                            lastItemNo++;
                            // debug output
                        }
                         // end for having more than 0 folder
                        // add our files
                        filesString = curFolderFile.list(DocSearch.wf);
                        numFiles = filesString.length;
                        for (int i = 0; i < numFiles; i++) {
                            // add them to our folderlist
                            curFi = curFolderString + DocSearch.pathSep + filesString[i];
                            curFi =  Utils.replaceAll(DocSearch.pathSep + DocSearch.pathSep, curFi, DocSearch.pathSep);
				curFileSizeBytes=Utils.getFileSize(curFi);
				if (curFileSizeBytes>ds.getMaxFileSize()) ds.setStatus(ds.dsFileTooBig+
					" ("+curFileSizeBytes+") "+filesString[i]);
			   else {
                            lastFound = indexNum(lastFound, curFi, ir);
                            if (lastFound == -1) {
                                ir.close();
                                // open writer to add document once again
                                iw = new IndexWriter(di.indexerPath, new StandardAnalyzer(), false);
                                addedSuccessFully = addDocToIndex(curFi, iw, di, di.isCdrom(), null);
                                switch (addedSuccessFully) {
                                case 1: // error
                                    errNum++;
                                    if (errNum < 8) {
                                        failedBuf.append("\n");
                                        failedBuf.append(curFi);
                                    }
                                    ds.setStatus(ds.dsErrIdxg+" " + curFi);
                                    break;
                                case 2: // meta robots = noindex
                                    numNoIndex++;
                                    if (numNoIndex < 8) {
                                        noRobotsBuf.append("\n");
                                        noRobotsBuf.append(curFi);
                                    }
                                    ds.setStatus("Document Exlusion (robots = NOINDEX) : "  + curFi);
                                    break;
                                default: // OK
                                    numNew++;
                                    ds.setStatus("New Document Added : "  + curFi);
                                    break;
                                } // end of switch
                                curSize++;
                                iw.close();
                                // reopen
                                ir = IndexReader.open(di.indexerPath);
                            } // end for lastfound not -1
			   } // end for file size not too big
                        }
                         // end for having more than 0 folder
                        // increment our curItem
                        folderList.set(curItemNo, null); // remove memory overhead as you go!
                        curItemNo++;
                        if (curSubNum >= maxSubNum) break;
                    } while (curItemNo <= lastItemNo);
                    //
                    ir.close(); // always close!
                    StringBuffer updateMSGBuf = new StringBuffer();
                    updateMSGBuf.append("\n");
                    updateMSGBuf.append(numRemovals);
                    updateMSGBuf.append(" files were removed from index.");
                    updateMSGBuf.append("\n");
                    updateMSGBuf.append(numUpdates);
                    updateMSGBuf.append(" files were reindexed.");
                    updateMSGBuf.append("\n");
                    updateMSGBuf.append(numNew);
                    updateMSGBuf.append(" new files were added to the index.");
                    updateMSGBuf.append("\n");
		    //
		    totalChanges=numRemovals+numUpdates+numNew;
		    // all our stuff to the notesBuf
		    addNote(updateMSGBuf.toString(), "", true);
		    // add our new and modified files
		    if (numNew>0) {
		    	addNote(Messages.getString("DocSearch.newFiles"), "", true);
		    	notesBuf.append(newItsBuf.toString());
			}
		    //
		    if (numUpdates>0) {
		    	addNote(Messages.getString("DocSearch.updatedFiles"), "", true);
		    	notesBuf.append(modItsItsBuf.toString());
			}
		    //
		    // 
		    if (numRemovals>0) {
		    	addNote(Messages.getString("DocSearch.deletedFiles"), "", true);
		    	notesBuf.append(delItsItsBuf.toString());
			}
		    //
		    addFooter();
                    if (errNum == 0) {
                        updateMSGBuf.append("No errors were encountered during this process.");
                        if (numNoIndex > 0) {
                            updateMSGBuf.append("\n\n" + numNoIndex
                                                + " files were not indexed due to meta data constraints (robots = NOINDEX), including:\n");
                            updateMSGBuf.append(noRobotsBuf.toString());
                        }
                        ds.showMessage("Update of index " + di.desc
                                       + " Completed", updateMSGBuf.toString());
                    }
                    else
                    {
                        updateMSGBuf.append(errNum+" errors were encountered during this process.\nThe following files had problems being indexed or re-indexed:\n"
                                            + failedBuf.toString());
                        if (numNoIndex > 0) {
                            updateMSGBuf.append("\n\n" + numNoIndex
                                                + " files were not indexed due to meta data constraints (robots = NOINDEX), including:\n");
                            updateMSGBuf.append(noRobotsBuf.toString());
                        }

                        ds.showMessage("Errors during Update of index "
                                       + di.desc, updateMSGBuf.toString());
                    } 
                }
                 // end of try
                catch (Exception eR) {
                    ds.showMessage("Error updating index " + di.desc, eR.toString());
                    eR.printStackTrace();
                }
                finally {
		    addFooter();
                    di.lastIndexed = DateTimeUtils.getToday();
                    ds.setStatus("Update of index " + di.desc + " completed.");
                }
            }
             // end if index is now from a zip archive
            else ds.doZipArchiveUpdate(di);
        }
    }

private String getFileType(String fileName) {
        String returnString = "unknown";
        int fileTypeEnding = fileName.lastIndexOf(".");
        if (fileTypeEnding != -1) returnString = fileName.substring(fileTypeEnding + 1, fileName.length());
        return returnString;
    }

private String getDocSummary(String fileName, int type) {
        String returnString = "Untitled";
        switch (type) {
        case 0: // HTML
            returnString = getHtmlSummary(fileName);
            break;
        case 1: // text
            returnString = getTextSummary(fileName);
            break;
        default: // all other
            returnString = getTextSummary(fileName);
            break;
        } // end for type
        return returnString;
    }

private String getTitle(String fileName) {
        int lastSlash = fileName.lastIndexOf(DocSearch.pathSep);
        boolean foundFileTitle = false;
        String newTitle = "Untitled";
        int fileLen = fileName.length();
        int fileTypeEnding = fileName.lastIndexOf(".");
        if (lastSlash != -1) {
            lastSlash++;
            if (fileTypeEnding > lastSlash) newTitle = fileName.substring(lastSlash, fileTypeEnding);
            else newTitle = fileName.substring(lastSlash, fileLen);
        }
        else {
            lastSlash = fileName.lastIndexOf("\\");
            if (lastSlash != -1) {
                lastSlash++;
                if (fileTypeEnding > lastSlash) newTitle = fileName.substring(lastSlash, fileTypeEnding);
                else newTitle = fileName.substring(lastSlash, fileLen);
            }
             // end for windows file or URL
        }
        if (newTitle.length() != 0) {
            newTitle = Utils.replaceAll("_", newTitle, " ").trim();
            foundFileTitle = true;
        }
        if (!foundFileTitle) return fileName;
        return newTitle;
    }

public String getTextSummary(String fileName) {
        String returnString = "No Summary";
        File file = new File(fileName);
        try {
            FileInputStream fi = new FileInputStream(file);
            InputStreamReader isr = new InputStreamReader(fi);
            Reader in = new BufferedReader(isr);
            int ch;
            int curI = 0; // reset i
            char curChar = ' ';
            byte curBint;
            int maxTitleLen = 286;
            int curCharNum = 0;
            char lastChar = ' ';
            boolean skipChar = false;
            int numLines = 0;
            StringBuffer titleBuf = new StringBuffer();
            while ((ch = in.read()) > -1)
            {
                curChar = (char)ch;
                // append to our title
                skipChar = false;
                if ((curChar == '\n') || (curChar == '\r')) curChar = ' ';
                if ((curChar == ' ') && (lastChar == ' '))  skipChar = true;
                if (!skipChar)  {
                    lastChar = curChar;
                    curCharNum++;
                    titleBuf.append(curChar);
                }
                if ((numLines > 3) || (curCharNum > maxTitleLen)) break;
            }
            String newTitle = titleBuf.toString().trim();
            if (newTitle.length() >= 4) returnString = newTitle + "...";
            else returnString = getTitle(fileName);
            fi.close();
            in.close();
        }
         // end of trying to catch file
        catch (Exception eF) {
            ds.setStatus("Error obtaining file title: " + fileName);
        }
        finally {
            return returnString;
        }
    }

 private String getHtmlSummary(String fileName) {
        String returnString = getMetaTag(fileName, "description");
        if (returnString.equals("")) {
            returnString = getMetaTag(fileName, "summary");
        }
        return returnString;
    }

private String getMetaTag(String fileName, String metaTag) {
        String lowerMetaTag = metaTag.toLowerCase();
        String returnString = "";
        File file = new File(fileName);
        try {
            FileInputStream fi = new FileInputStream(file);
	    InputStreamReader isr = new InputStreamReader(fi);
            Reader in = new BufferedReader(isr);
            int curI = 0; // reset i
            char curChar = ' ';
            byte curBint;
            int maxTitleLen = 36;
            int curCharNum = 0;
            char lastChar = ' ';
            boolean skipChar = false;
            int numLines = 0;
            StringBuffer tagBuf = new StringBuffer();
            boolean inTag = false;
            boolean inTitle = false;
            String tagString = "";
            String lowerTag = "";
            String attr = "";
            String lowerAttr = "";
            while (curI != -1) {
                curI = in.read();
                if (curI != -1) {
                    curChar = (char)curI;
                    // append to our title
                    skipChar = false;
                    if ((curChar == '\n') || (curChar == '\r')) curChar = ' ';
                    curCharNum++;
                    if (curChar == '<') inTag = true;
                    if (curChar == '>') {
                        tagBuf.append(curChar);
                        inTag = false;
                        tagString = tagBuf.toString();
                        lowerTag = tagString.toLowerCase();
                        if (lowerTag.startsWith("<meta")) {
                            attr = Utils.getTagString("name=", tagString);
                            lowerAttr = attr.toLowerCase().trim();
                            if (lowerAttr.equals(lowerMetaTag)) {
                                returnString =
                                    Utils.getTagString("content=", tagString);
                                System.out.println(fileName + " has "
                                                   + returnString + "\n for a "
                                                   + lowerMetaTag);
                                break;
                            }
                        }
                        if (lowerTag.startsWith("<body")) break;
                        tagBuf = new StringBuffer();
                    }
                    if ((curChar == ' ') && (lastChar == ' ')) skipChar = true;
                    if (!skipChar) lastChar = curChar;
                    if (inTag) tagBuf.append(curChar);
                }
                else break;
            }

            fi.close();
	    isr.close();
	    in.close();
        }
        catch (Exception eF) {
            ds.setStatus("Error obtaining file author: " + fileName);
        }
        finally {
            return returnString;
        }
    }

private String getAuthor(String fileName) {
        String returnString = "";
        File file = new File(fileName);
        try {
            FileInputStream fi = new FileInputStream(file);
	    InputStreamReader isr = new InputStreamReader(fi);
            Reader in = new BufferedReader(isr);
            int curI = 0; // reset i
            char curChar = ' ';
            byte curBint;
            int maxTitleLen = 36;
            int curCharNum = 0;
            char lastChar = ' ';
            boolean skipChar = false;
            int numLines = 0;
            StringBuffer tagBuf = new StringBuffer();
            boolean inTag = false;
            boolean inTitle = false;
            String tagString = "";
            String lowerTag = "";
            String attr = "";
            String lowerAttr = "";
            while (curI != -1) {
                curI = in.read();
                if (curI != -1) {
                    curChar = (char)curI;
                    // append to our title
                    skipChar = false;
                    if ((curChar == '\n') || (curChar == '\r')) curChar = ' ';
                    curCharNum++;
                    if (curChar == '<') inTag = true;
                    if (curChar == '>') {
                        tagBuf.append(curChar);
                        inTag = false;
                        tagString = tagBuf.toString();
                        lowerTag = tagString.toLowerCase();
                        if (lowerTag.startsWith("<meta")) {
                            attr = Utils.getTagString("name=", tagString);
                            lowerAttr = attr.toLowerCase();
                            if ((lowerAttr.indexOf("author") != -1)
                                    || (lowerAttr.indexOf("webmaster") != -1)) {
                                returnString = Utils.getTagString("content=", tagString);
                                break;
                            }
                        }
                        if (lowerTag.startsWith("<body")) break;
                        tagBuf = new StringBuffer();
                    }
                    if ((curChar == ' ') && (lastChar == ' ')) skipChar = true;
                    if (!skipChar) lastChar = curChar;
                    if (inTag) tagBuf.append(curChar);
                }
                else break;
            }
            fi.close();
	    in.close();
	    isr.close();
        }
        catch (Exception eF) {
            ds.setStatus("Error obtaining file author: " + fileName);
        }
        finally {
            return returnString;
        }
    }

private String getDocTitle(String fileName, int type) {
        String returnString = "Untitled";
        switch (type) {
        case 0: // HTML
            returnString = getHtmlTitle(fileName);
            break;
        case 1: // text
            returnString = getTextTitle(fileName);
            break;
        case 6: // oo
            returnString = getTextTitle(fileName);
            break;
        case 7: // oo
            returnString = getTextTitle(fileName);
            break;
        case 8: // oo
            returnString = getTextTitle(fileName);
            break;
        case 9: // oo
            returnString = getTextTitle(fileName);
            break;
        default: // all other
            returnString = getTitle(fileName);
            break;
        } // end for type
        return returnString;
    }

private String getHtmlTitle(String fileName) {
        String returnString = "Untitled";
        File file = new File(fileName);
        try {
	    FileInputStream fi = new FileInputStream(file);
	    InputStreamReader isr = new InputStreamReader(fi);
            Reader in = new BufferedReader(isr);
            int curI = 0; // reset i
            char curChar = ' ';
            byte curBint;
            int maxTitleLen = 36;
            int curCharNum = 0;
            char lastChar = ' ';
            boolean skipChar = false;
            int numLines = 0;
            StringBuffer tagBuf = new StringBuffer();
            boolean inTag = false;
            boolean inTitle = false;
            String tagString = "";
            String lowerTag = "";
            StringBuffer titleBuf = new StringBuffer();
            while (curI != -1) {
                curI = in.read();
                if (curI != -1) {
                    curChar = (char)curI;
                    // append to our title
                    skipChar = false;
                    if ((curChar == '\n') || (curChar == '\r')) curChar = ' ';
                    curCharNum++;
                    if (curChar == '<') {
                        inTag = true;
                        if (inTitle) break;
                    }
                    if (curChar == '>') {
                        tagBuf.append(curChar);
                        inTag = false;
                        tagString = tagBuf.toString();
                        lowerTag = tagString.toLowerCase();
                        if (lowerTag.startsWith("<titl")) inTitle = true;
                        else if (lowerTag.startsWith("</head")) break;
                        else if (lowerTag.startsWith("<body")) break;
                        tagBuf = new StringBuffer();
                    }
                     // end for end of a tag

                    if ((curChar == ' ') && (lastChar == ' ')) skipChar = true;
                    if (!skipChar) lastChar = curChar;
                    if (inTag) tagBuf.append(curChar);
                    else if (inTitle) {
                        if ((!skipChar) && (curChar != '>')) titleBuf.append(curChar);
                    }
                }
                else break;
            }
            fi.close();
	    in.close();
	    isr.close();
            String newTitle = titleBuf.toString().trim();
            if (newTitle.length() >= 4) returnString = newTitle;
        }
         // end of trying to catch file
        catch (Exception eF) {
            ds.setStatus("Error obtaining file title: " + fileName);
        }
        finally {
            return returnString;
        }
    }

private String getTextTitle(String fileName) {
        String returnString = "Untitled";
        File file = new File(fileName);
        try {
            FileInputStream fi = new FileInputStream(file);
	    InputStreamReader isr = new InputStreamReader(fi);
            Reader in = new BufferedReader(isr);
            int curI = 0; // reset i
            char curChar = ' ';
            byte curBint;
            int maxTitleLen = 36;
            int curCharNum = 0;
            char lastChar = ' ';
            boolean skipChar = false;
            int numLines = 0;
            StringBuffer titleBuf = new StringBuffer();
            while (curI != -1) {
                curI = in.read();
                if (curI != -1) {
                    curChar = (char)curI;
                    // append to our title
                    skipChar = false;
                    if ((curChar == '\n') || (curChar == '\r')) curChar = ' ';
                    if ((curChar == ' ') && (lastChar == ' ')) skipChar = true;
                    if (!skipChar) {
                        lastChar = curChar;
                        curCharNum++;
                        titleBuf.append(curChar);
                    }
                    if ((numLines > 3) || (curCharNum > maxTitleLen)) break;
                }
                else break;
            }
            String newTitle = titleBuf.toString().trim();
            if (newTitle.length() >= 4) returnString = newTitle + "...";
            else returnString = getTitle(fileName);
            fi.close();
	    in.close();
	    isr.close();
        }
        catch (Exception eF) {
            ds.setStatus("Error obtaining file title: " + fileName);
        }
        finally {
            return returnString;
        }
    }

private void removeAllTags(String originalFile, String newFileName) throws IOException {
        boolean inTag = false;
        File origFile = new File(originalFile);
        FileInputStream fi = new FileInputStream(origFile);
        InputStreamReader isr = new InputStreamReader(fi);
        FileWriter filewriter = new FileWriter(newFileName);
        PrintWriter pw = new PrintWriter(filewriter);
        StringBuffer tagBuf = new StringBuffer();
        StringBuffer nonTagTextf = new StringBuffer();
        String t = "";
        int curI = 0; // reset i
        byte rB;
        byte curBint;
        char curChar = ' ';
        Reader in = new BufferedReader(isr);
        int ch;
        while ((ch = in.read()) > -1) {
            curChar = (char)ch;
            if (curChar == '>') {
                inTag = false;
                nonTagTextf = new StringBuffer();
            }
            else if (curChar == '<') {
                inTag = true;
                t = nonTagTextf.toString().trim();
                if (t.length() > 0)
                {
                    pw.println(t);
                }
            }
            if ((!inTag) && (curChar != '>')) {
                nonTagTextf.append(curChar);
            }
        }

        fi.close();
        in.close();
        filewriter.close();
        pw.close();
    }

private String getTagText(String tagPrefix, String fileName) throws IOException {
        tagPrefix = tagPrefix.toLowerCase();
        String tagStart = "<" + tagPrefix;
        String tagEnd = "</";
        StringBuffer retBuf = new StringBuffer();
        File origFile = new File(fileName);
        FileInputStream fi = new FileInputStream(origFile);
	InputStreamReader isr = new InputStreamReader(fi);
	Reader in = new BufferedReader(isr);
        int curI = 0; // reset i
        byte rB;
        byte curBint;
        char curChar = ' ';
        StringBuffer tagBuf = new StringBuffer();
        StringBuffer nonTagTextf = new StringBuffer();
        boolean readContent = false;
        boolean inTag = false;
        String tagStr = "";
        while (curI != -1) {
            curI = in.read();
            if (curI != -1) {
                curBint = (byte)curI;
                curChar = (char)curI;
                if (curChar == '>') {
                    tagStr = tagBuf.toString().toLowerCase();
                    if 	((tagStr.indexOf("/")==-1) && (tagStr.indexOf(tagPrefix)!=-1))
                    {  // (tagStr.startsWith(tagStart))
                        readContent = true;
                    }
                    if ((tagStr.startsWith(tagEnd)) && (tagStr.indexOf(tagPrefix)!=-1)) {
                        retBuf.append(nonTagTextf.toString());
                        System.out.println("Value for " + tagPrefix + " is "
                                           + nonTagTextf.toString() + " in "
                                           + fileName);
                        break;
                    }
                    tagBuf = new StringBuffer();
                    inTag = false;
                }
                else if (curChar == '<') inTag = true;
                if (inTag) tagBuf.append(curChar);
                else if ((readContent) && (curChar != '>')) nonTagTextf.append(curChar);
                else if ((readContent) && (curChar == '>')) nonTagTextf.append(" ");
            }
            else break;
        }
        fi.close();
	isr.close();
	in.close();
        return retBuf.toString();
    }
    



public StringBuffer getUpDateNotes() {
	return notesBuf;
    }
    
public void addNote(String message, String link, boolean newLine) {
    boolean useLink=true;
    if (link.equals("")) useLink=false;
    if (doEmail) {
    	if (isTextEmailFormat) {
	    	notesBuf.append(message);
		} // end of text format
	else {
		if (newLine) notesBuf.append("<p align=\"left\">");
		message=Utils.replaceAll("\n", message, "<br>");
		if (useLink) {
			notesBuf.append("<a href=\"");
			notesBuf.append(link);
			notesBuf.append("\">");
			}
		notesBuf.append(message);
		if (useLink) notesBuf.append("</a>");
		if (newLine) notesBuf.append("</p>");
		}
	if (newLine) notesBuf.append("\n");
	}
}

public void setDoEmail(boolean toSet) {
	doEmail=toSet;
}

public void addFooter() {
    if (doEmail) {
    	if (isTextEmailFormat) {
		notesBuf.append("\n");
		} // end of text format
	else {
		notesBuf.append("</BODY></HTML>");
		notesBuf.append("\n");
		}
	}
}

public void addHeader(String title) {
    if (doEmail) {
    	if (isTextEmailFormat) {
	    	notesBuf.append(title);
		notesBuf.append("\n");
		} // end of text format
	else {
		notesBuf.append("<HTML><HEAD><TITLE>");
		notesBuf.append(title);
		notesBuf.append("</TITLE></HEAD><BODY><h3>");
		notesBuf.append(title);
		notesBuf.append("</h3>");
		notesBuf.append("\n");
		}
	}
}
        
public void setEmailText(boolean isEmail) {
	isTextEmailFormat=isEmail;
}
 
public void addToSummary(String title, String author, String dateStr, 
	String lowerType, String curSummary, String urlStr, String curFileSize) {
	if (debug) {
		//if (lowerType.startsWith("pdf")) 
		System.out.println("\n   "+title+"\n   "+curSummary+"\n  "+urlStr+"  "+lowerType);
		}
	int curInsrtMd=getInsertMode();
	if (doEmail) {
		// TEXT
		if (isTextEmailFormat) { // 0 =new, 1= mod
			switch (curInsrtMd) {
				case 1: //
					modItsItsBuf.append("\n");
					modItsItsBuf.append(title);
					modItsItsBuf.append("\n");
					modItsItsBuf.append(curSummary);
					modItsItsBuf.append("\n");
					modItsItsBuf.append(urlStr);
					modItsItsBuf.append("\n");
					modItsItsBuf.append(author);
					modItsItsBuf.append(", ");
					modItsItsBuf.append(curFileSize);
					modItsItsBuf.append(", ");
					modItsItsBuf.append(lowerType);
					modItsItsBuf.append("\n");
					break;
				default: // new
					newItsBuf.append("\n");
					newItsBuf.append(title);
					newItsBuf.append("\n");
					newItsBuf.append(curSummary);
					newItsBuf.append("\n");
					newItsBuf.append(urlStr);
					newItsBuf.append("\n");
					newItsBuf.append(author);
					newItsBuf.append(", ");
					newItsBuf.append(curFileSize);
					newItsBuf.append(", ");
					newItsBuf.append(lowerType);
					newItsBuf.append("\n");
					break;
				}
			}
		else {  
		// HTML
			switch (curInsrtMd) {
				case 1: //
					modItsItsBuf.append("<p align=\"left\"><a href=\"");
					modItsItsBuf.append(urlStr);
					modItsItsBuf.append("\"><b>");
					modItsItsBuf.append(title);
					modItsItsBuf.append("</b></a><br>");
					modItsItsBuf.append(curSummary);
					modItsItsBuf.append("<br>");
					modItsItsBuf.append(urlStr);
					modItsItsBuf.append("<br>");
					modItsItsBuf.append(author);
					modItsItsBuf.append(", ");
					modItsItsBuf.append(curFileSize);
					modItsItsBuf.append(", ");
					modItsItsBuf.append(lowerType);
					modItsItsBuf.append("</p>");
					break;
				default: // new
					newItsBuf.append("<p align=\"left\"><a href=\"");
					newItsBuf.append(urlStr);
					newItsBuf.append("\"><b>");
					newItsBuf.append(title);
					newItsBuf.append("</b></a><br>");
					newItsBuf.append(curSummary);
					newItsBuf.append("<br>");
					newItsBuf.append(urlStr);
					newItsBuf.append("<br>");
					newItsBuf.append(author);
					newItsBuf.append(", ");
					newItsBuf.append(curFileSize);
					newItsBuf.append(", ");
					newItsBuf.append(lowerType);
					newItsBuf.append("</p>");
					break;
				}
			}
		}
}

public int getInsertMode() {
	return insertMode;
}

public void addDelNote(org.apache.lucene.document.Document doc) {
	int curInsrtMd=getInsertMode();
	String title=doc.get("title");
	String author=doc.get("author");
	String urlStr=doc.get("url");
	String curSummary=doc.get("summary");
	String curFileSize=doc.get("size");
	String lowerType=doc.get("type");
	if (doEmail) {
		// TEXT
		if (isTextEmailFormat) { // 0 =new, 1= mod
			delItsItsBuf.append("\n");
			delItsItsBuf.append(title);
			delItsItsBuf.append("\n");
			delItsItsBuf.append(curSummary);
			delItsItsBuf.append("\n");
			delItsItsBuf.append(urlStr);
			delItsItsBuf.append("\n");
			delItsItsBuf.append(author);
			delItsItsBuf.append(", ");
			delItsItsBuf.append(curFileSize);
			delItsItsBuf.append(", ");
			delItsItsBuf.append(lowerType);
			delItsItsBuf.append("\n");
			} // end for text
		else { // html email format
			delItsItsBuf.append("<p align=\"left\"><a href=\"");
			delItsItsBuf.append(urlStr);
			delItsItsBuf.append("\"><b>");
			delItsItsBuf.append(title);
			delItsItsBuf.append("</b></a><br>");
			delItsItsBuf.append(curSummary);
			delItsItsBuf.append("<br>");
			delItsItsBuf.append(urlStr);
			delItsItsBuf.append("<br>");
			delItsItsBuf.append(author);
			delItsItsBuf.append(", ");
			delItsItsBuf.append(curFileSize);
			delItsItsBuf.append(", ");
			delItsItsBuf.append(lowerType);
			delItsItsBuf.append("</p>");
			}
		} // end for doEmail
}

public void setInsertMode(int toSet) {
	insertMode=toSet;
}

public int getTotalChanges() {
	return totalChanges;
}

public WebPageMetaData getWebPageMetaData(String fileName) {
	WebPageMetaData tempWpmd=new WebPageMetaData(fileName);
	try {
		boolean inTag = false;
		boolean foundSummary=false;
		boolean inBody=false;
		boolean foundDesc=false;
		boolean inScript=false;
		boolean inTitle=false;
		StringBuffer tagBuf=new StringBuffer();
		StringBuffer titleBuf=new StringBuffer();
		StringBuffer sumBuf=new StringBuffer();
		String realTag="";
		String lowerTag="";
        	File origFile = new File(fileName);
        	FileInputStream fi = new FileInputStream(origFile);
        	InputStreamReader isr = new InputStreamReader(fi);
		Reader in = new BufferedReader(isr);
        	FileWriter filewriter = new FileWriter(ds.htmlTextFile);
        	PrintWriter pw = new PrintWriter(filewriter);
        	StringBuffer nonTagTextf = new StringBuffer();
        	String t = ""; String tempMetaName="";
		String tempMetaContent="";
		int curBodyNonTagTextNum=0;
        	int curI = 0; byte rB;
		int tSize=0; int sumMaxSize=220;
        	byte curBint; char curChar = ' ';
        	int ch;
		while ((ch = in.read()) > -1) {
            	curChar = (char)ch;
            	if (curChar == '>') {
	                inTag = false;
			//
			tagBuf.append(curChar);
			realTag=tagBuf.toString();
			lowerTag=realTag.toLowerCase();
			if (lowerTag.startsWith(META_TAG)) {
				tempMetaName=Utils.getTagString("name=", lowerTag);
				if (tempMetaName.startsWith("description")) {
					tempMetaContent=Utils.getTagString("content=", realTag);
					if (!tempMetaContent.trim().equals(""))  {
						tempWpmd.setDesc(tempMetaContent);
						foundSummary=true;
						}
					}
				else if (tempMetaName.startsWith("summary")) {
					tempMetaContent=Utils.getTagString("content=", realTag);
					if (!tempMetaContent.trim().equals(""))  {
						tempWpmd.setDesc(tempMetaContent);
						foundSummary=true;
						}
					}
				else if ((tempMetaName.startsWith("author")) ||
					(tempMetaName.indexOf("webmaster")!=-1)) {
					tempMetaContent=Utils.getTagString("content=", realTag);
					tempWpmd.setAuthor(tempMetaContent);
					}
				}
			else if (lowerTag.startsWith(SCRIPT_TAG)) {
				if (!lowerTag.endsWith("/>")) inScript=true;
				}
			else if (lowerTag.startsWith(SCRIPT_TAG_END)) {
				inScript=false;
				}
			else if (lowerTag.startsWith(BODY_TAG)) {
				inBody=true;
				}
			else if (lowerTag.startsWith(BODY_TAG_END)) {
				inBody=false;
				}
			else if (lowerTag.startsWith(TITLE_TAG)) {
				inTitle=true;
				}
			else if (lowerTag.startsWith(TITLE_TAG_END)) {
				inTitle=false;
				tempWpmd.setTitle(titleBuf.toString());
				}
			// reset our buffers
			tagBuf = new StringBuffer();
			nonTagTextf = new StringBuffer();
            	}
            	else if (curChar == '<') {
	                inTag = true;
			tagBuf = new StringBuffer();
                	t = nonTagTextf.toString().trim();
			tSize=t.length();
                	if (tSize > 0) {
				if ((!inScript) && (inBody)) pw.println(t);
				}
			nonTagTextf = new StringBuffer();
			//
			if (!foundSummary) {
				// 
				if (inBody) {
					curBodyNonTagTextNum+=tSize;
					sumBuf.append(" ");
					sumBuf.append(t);
					sumBuf.append(" ");
					if (curBodyNonTagTextNum>=sumMaxSize) {
						tempWpmd.setDesc(Utils.concatStrToEnd(sumBuf.toString(), sumMaxSize));
						foundSummary=true;
						}
					}
				}
			//
            	} // end for the beginning of a tag
		if ((inTitle) && (curChar != '>') && (!inTag)) titleBuf.append(curChar);
            	else if ((!inTag) && (curChar != '>')) nonTagTextf.append(curChar);
		else if (inTag) tagBuf.append(curChar);
        	} // end for while reading
		if ((!foundSummary) && (curBodyNonTagTextNum>0)) {
			tempWpmd.setDesc(sumBuf.toString());
			}
	        fi.close();
	        in.close();
	        filewriter.close();
	        pw.close();
		//
	} // end for try
	catch (Exception eR) {
		ds.setStatus(ds.dsErr+" : "+eR.toString());
		}
	finally {
		return tempWpmd;
		}
} // end for getWebPageMetaData

public void doSpiderUpdate(DocSearcherIndex idx) {
	/* updates an index that is spider based */
	int maxDocsToGet=idx.depth;
	int percentGrowth=10;
	int pcntNum=maxDocsToGet/percentGrowth;
	/* allow spidered indexes to grow by 10 percent */
	if (pcntNum>0) maxDocsToGet+=pcntNum;
	ds.setStatus(ds.plsWait+" "+ds.waitForIndexUpDate+" "+idx.getDesc());
	/* load the list of previously found links */
	String linksListName=Utils.addFolder(ds.indexDir, Utils.replaceAll(" ", idx.getDesc(), "_")+".txt");
	ArrayList oldSpiderLinks=Utils.getSpiderLinks(linksListName);
	System.out.println("Previously found link num total="+oldSpiderLinks.size());
	ArrayList newLinks=new ArrayList();
	ArrayList changedLinks=new ArrayList();
	ArrayList deleteLink=new ArrayList();
	int numDeletes=0; int numChanges=0; int numNew=0; int numUnChanged=0;
	int numMetaNoIdx=0; int numFails=0; int numTotalDocs=0;
	LinkFinder ulf=new LinkFinder (idx.path, 
		linksListName, 
		idx.depth, 
		ds, 
		idx,   
		oldSpiderLinks);
	ulf.init();
	try {
		ulf.update();
		}
	catch (Exception eR) {
		ds.showMessage(ds.dsErr, eR.toString());
		eR.printStackTrace();
		}
	finally {
		numNew=ulf.getNumNew();
		numDeletes=ulf.getNumDeletes();
		numMetaNoIdx=ulf.getNumMetaNoIdx();
		numChanges=ulf.getNumUpdates();
		numUnChanged=ulf.getNumUnchanged();
		numFails=ulf.getNumFails();
		//ds.setStatus(ds.dsErr+" "+eR.toString());
		StringBuffer resultsMessage=new StringBuffer();
		resultsMessage.append(numNew+" "+ds.dsNumNewFiles+"\n\n");
		resultsMessage.append(numDeletes+" "+ds.dsNumDelFiles+"\n\n");
		resultsMessage.append(numChanges+" "+ds.dsNumchangedFiles+"\n\n");
		resultsMessage.append(numUnChanged+" "+ds.dsNumUnchangedFiles+"\n\n");
		resultsMessage.append(numMetaNoIdx+" "+ds.dsNotIdxdMeta+"\n\n");
		resultsMessage.append(numFails+" "+ds.dsFailIdxDocs+"\n\n");
		numTotalDocs=numUnChanged+numNew-numFails;
		resultsMessage.append(numTotalDocs+" "+ds.dsTtlDxInIdx+"\n\n");
		ds.showMessage(idx.getDesc()+" "+ds.dsUpdts, resultsMessage.toString());
		}
} // end for doSpiderUpdate

} // end for class
