import java.util.*;
import java.io.*;
import java.net.*;
import javax.net.ssl.*;
import java.net.URL;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;

public class LinkFinder {
	int numSkips=0;
	DocSearch ds=null;
	IndexWriter iw; IndexReader ir;
	int numDeletes=0; int numChanges=0; int numNew=0; int numUnChanged=0;
	int numMetaNoIdx=0; int numFails=0; 
	long maxFileSizeToGet=600000;
	long curModified=0; long curFiSi=0; String curContentType="";
	String pageName=""; String downloadFile="dnldfi.tmp";
	String downloadFileDir=System.getProperty("java.io.tmpdir");
	DocSearcherIndex dsi;
	String baseUrlFolder="";
	String domainUrl="";
	String outFile="";
	ArrayList links=new ArrayList();
	ArrayList skippedLinks=new ArrayList();
	String [] htmlTypes={ ".htm", ".shtm", ".html", ".shtm", ".asp", ".php", ".jsp", ".cfm", ".cfml",
				".do", ".aspx", ".jhtml", "/"};
	String [] nonHtmlTypes={ ".zip", ".jpg", ".bmp", ".gif", ".db", ".cat", ".wmf", ".tif", ".tiff", ".swf", 
				".ncd", ".pdd", ".png", ".ppt", ".jpeg", ".mdb", ".msg", ".mpp", ".log"};
	String [] bogusDirs={ "_vti_", "_private", "file:"};
	final static String DOWNLOADING=Messages.getString("DocSearch.dnldg");
	final static String DOWNLOAD_COMPLETE=Messages.getString("DocSearch.dnldComplete");
	final static String ERROR=Messages.getString("DocSearch.err");
	final static String SKIPPING_LINK_TOO_LONG=Messages.getString("DocSearch.lnkTooBig");
	final static String K_BYTES=Messages.getString("DocSearch.bytes");
	final static String LINKS_FOUND=Messages.getString("DocSearch.lnxFnd");
	final static String DOCS_IDXD=Messages.getString("DocSearch.doxIdxd");
	final static String SKIPPING_LINK=Messages.getString("DocSearch.skippingLink");
	final static String SUCCESSFUL_DOWNLOADS=Messages.getString("DocSearch.successfulSpiders");
	final static String FAILED_DOWNLOADS=Messages.getString("DocSearch.failedSpiders");
	final static String META_DOWNLOADS=Messages.getString("DocSearch.nonMetaSpiders");
	final static String CONNECTING_TO=Messages.getString("DocSearch.cnktgTo");
	final static String SPIDERING_COMPLETE=Messages.getString("DocSearch.spideringComplete");
	boolean restrictToSite=true;
	int maxLinksToFind=5000;
	String statusString="";
	boolean isDsReady=false;
	
LinkFinder (String pageName, String outFile) {
	this.pageName=pageName;
	this.outFile=outFile;
} // end for constructor

LinkFinder (String pageName, String outFile, int maxLinksToFind, 
	DocSearch ds, DocSearcherIndex dsi, IndexWriter iw) {
	this.pageName=pageName;
	this.outFile=outFile;
	this.maxLinksToFind=maxLinksToFind;
	this.ds=ds;
	this.dsi=dsi;
	this.iw=iw;
	isDsReady=true;
	downloadFileDir=ds.tempDir;
	maxFileSizeToGet=ds.getMaxFileSize();
} // end for constructor

LinkFinder (String pageName, 
	String outFile, 
	int maxLinksToFind, 
	DocSearch ds, 
	DocSearcherIndex dsi,   
	ArrayList links) {
	this.pageName=pageName;
	this.outFile=outFile;
	this.maxLinksToFind=maxLinksToFind;
	this.ds=ds;
	this.dsi=dsi;
	this.links=links;
	isDsReady=true;
	downloadFileDir=ds.tempDir;
	maxFileSizeToGet=ds.getMaxFileSize();
} // end for constructor

public void init() {
    // Create a trust manager that does not validate certificate chains
    TrustManager[] trustAllCerts = new TrustManager[]{
        new X509TrustManager() {
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return null;
            }
            public void checkClientTrusted(
                java.security.cert.X509Certificate[] certs, String authType) {
            }
            public void checkServerTrusted(
                java.security.cert.X509Certificate[] certs, String authType) {
            }
        }
    };
    
    // Install the all-trusting trust manager
    try {
        SSLContext sc = SSLContext.getInstance("SSL");
        sc.init(null, trustAllCerts, new java.security.SecureRandom());
        HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
    } catch (Exception e) {
    }
}

public static void main (String [] args) {

	LinkFinder lf=new LinkFinder(args[0], args[1]);
	lf.init();
	lf.getAllLinks();
}

public boolean isHtml(String toCheck) {
	boolean returnBool=false;
	String lowerUrl=toCheck.toLowerCase();
	int numHtmlTypes=htmlTypes.length;
	for (int i=0;i<numHtmlTypes;i++) {
		if (lowerUrl.endsWith(htmlTypes[i])) {
			returnBool=true;
			break;
			}
		} // end for types
	return returnBool;
}



public boolean skippType(String toCheck) {
	boolean returnBool=false;
	String lowerUrl=toCheck.toLowerCase();
	int numNonHtmlTypes=nonHtmlTypes.length;
	for (int i=0;i<numNonHtmlTypes;i++) {
		if (lowerUrl.endsWith(nonHtmlTypes[i])) {
			returnBool=true;
			break;
			}
		} // end for types
	return returnBool;
}

/* check for bogusDirs */
public boolean hasBogusDirs(String toCheck) {
	boolean returnBool=false;
	String lowerUrl=toCheck.toLowerCase();
	int numNonHtmlTypes=bogusDirs.length;
	for (int i=0;i<numNonHtmlTypes;i++) {
		if (lowerUrl.indexOf(bogusDirs[i])!=-1) {
			returnBool=true;
			break;
			}
		} // end for types
	return returnBool;
}

public int getNextUrlNo() {
	int returnInt=-1;
	SpiderUrl spy=null;
	Iterator it=links.iterator();
	int curPos=0;
	while (it.hasNext()) {
		spy=(SpiderUrl)it.next();
		if (!spy.spidered()) {
			if (!spy.isDeadLink()) {
				returnInt=curPos;
				break;
				}
			}
		curPos++;
		}
	return returnInt;
}

public String getLinkNameByNo(int toGet) {
	SpiderUrl spy=(SpiderUrl)links.get(toGet);
	return spy.getUrl();
}

public SpiderUrl getSpiderUrl(int toGet) {
	SpiderUrl spy=(SpiderUrl)links.get(toGet);
	return spy;
}

public void getAllLinks() {
	/* writes links from a page out to a file */
	String urlStr=pageName; String shortUrl="";
	numUnChanged=0;
	int numSuccesses=0; int numFailed=0; int numNoRobots=0;
	addLink(urlStr);
	String dmUrl=Utils.getDomainUrl(urlStr);
	domainUrl=dmUrl;
	numSkips=0; SpiderUrl curl=new SpiderUrl(urlStr);
	baseUrlFolder=Utils.getBaseUrlFolder(urlStr);
	System.out.println("DOMAIN URL: "+domainUrl);
	int curLinkNo=0;
	String dnldTmpName="download.tmp"; String curMd5="";
	boolean curIsWebPage=true;
	boolean completedSpider=false;
	String appletParam="";
	boolean isDead=false;
	int numSpidered=0;
	String errStr=""; int curSuccessNo=0;
	while (curLinkNo!=-1) {
		try {
			completedSpider=false; isDead=false;
			errStr="";
			curLinkNo=getNextUrlNo();
			if (curLinkNo==-1) {
				System.out.println("End of links reached.");
				break;
				}
			else {
				urlStr=getLinkNameByNo(curLinkNo);
				System.out.println("ANALYZING PAGE "+urlStr);
				curl=getSpiderUrl(curLinkNo);
				}
			InputStream urlStream;
			URL url=new URL(urlStr); 
			shortUrl=Utils.concatEnd(urlStr, 33);
			setStatus(CONNECTING_TO+" "+shortUrl);
			URLConnection conn =(URLConnection)url.openConnection();
			FileOutputStream dos;
			conn.setDoInput(true);
			conn.setUseCaches(false);
			conn.connect();
			urlStream=conn.getInputStream();
			int fileS=conn.getContentLength();
			if (fileS>maxFileSizeToGet) {
				String ex=SKIPPING_LINK_TOO_LONG+" ("+fileS+" > "+maxFileSizeToGet+") "+shortUrl;
			        setStatus(ex);
				throw new Exception(ex);
				}
			setStatus(DOWNLOADING+" "+shortUrl+" "+fileS+" "+K_BYTES);
			curFiSi=(long)fileS;
			curl.setSize(curFiSi);
			curModified=conn.getLastModified();  // was .getDate();
			curl.setLastModified(curModified);
			curContentType=conn.getContentType();
			if (curContentType!=null) curl.setContentType(curContentType);
			/* build the value for downloadFile */
			dnldTmpName=ds.getDownloadFileName(curl.getContentType(), urlStr.toLowerCase());
			downloadFile=Utils.addFolder(downloadFileDir, dnldTmpName);
			curIsWebPage=isHtml(urlStr.toLowerCase());
			System.out.println("saving to "+downloadFile);
			File saveFile=new File(downloadFile);
			dos = new FileOutputStream(saveFile);
			int curSize=-1; byte curBint; int curI=0;
			int curPercent=0; int lastPercent=0;
			curSize++;
			char c=' '; 
			StringBuffer tagBuf=new StringBuffer();
			StringBuffer contentBuf=new StringBuffer();
			String realTag=""; String lowerTag="";
			String link=""; String desc=""; long size=0;
			String fullUrl=""; String type="";
			String content=""; boolean inTag=false;
			String realUrl="";
			LinkValue lv;
			while(curI!=-1) {
				curI=urlStream.read();
				if (curI==-1) break;
				curBint=(byte)curI;
				dos.write(curBint);
				curSize++;
				/* notify of download progress */
				if ((curSize>0) && (curSize%10==0)) {
					curPercent=(curSize*100)/fileS;
					if (curPercent!=lastPercent) {
						lastPercent=curPercent;
						setStatus(DOWNLOADING+" : ("+shortUrl+") --> "
							+curPercent+" %"+" ( "
							+(numSuccesses+numFailed+numNoRobots)                   
							+"/"+getNumLinksFound()+")");
						}
					} // end for percent updates
				else  if (curSize%40==0) setStatus(DOWNLOADING
					+" : ("+shortUrl+") --> "
					+curSize+" "+K_BYTES);
				/* handle links */
				if (curIsWebPage) {
					c=(char)curBint;
					// LOOK AT THE TAGS
					if (c=='<') {
						inTag=true;
						tagBuf=new StringBuffer();
						} // end for start of a tag	
					else if (c=='>') {
						tagBuf.append(c);
						realTag=tagBuf.toString();
						lowerTag=realTag.toLowerCase();
						if (lowerTag.startsWith("</a")) {
							/* desc=Utils.concat(Utils.htmlToText(contentBuf.toString()), 36)+"...";
							if (restrictToSite) {
								if ((link.startsWith(dmUrl)) || (link.startsWith("../")) || (link.startsWith("/"))) doPossibleAdd(pageName, link);
								else System.out.println(link+" SKIPPED - doesn't start with "+dmUrl);
								}
							else */
							doPossibleAdd(pageName, link);
							}
						else if (lowerTag.startsWith("<area")) {
							link=Utils.getTagString("href=", realTag);
							link=Utils.getNormalUrl(link);
							if (restrictToSite) {
								doPossibleAdd(pageName, link);
								}
							else doPossibleAdd(pageName, link);
							}
						else if (lowerTag.startsWith("<param")) {
							appletParam=Utils.getTagString("name=", realTag);
							if (appletParam.toLowerCase().equals("url")) {
								link=Utils.getTagString("value=", realTag);
								link=Utils.getNormalUrl(link);
								if (restrictToSite) {
									doPossibleAdd(pageName, link);
									}
								else doPossibleAdd(pageName, link);
								}
							}
						else if (lowerTag.startsWith("<a href=")) {
							link=Utils.getTagString("href=", realTag);
							/* remove funky formatting */
							link=Utils.getNormalUrl(link);
							//
							contentBuf=new StringBuffer();
							}
						inTag=false;
						} // end for end of a tag
					//
					if (inTag) tagBuf.append(c);
					else if ((c!='>') && (c!='>')) contentBuf.append(c);
					} // end for isWebPage
				/* past our limit yet? */
				if (numSpidered>maxLinksToFind) break;
			} // end while downloading
			if (numSpidered>maxLinksToFind) break;
			dos.close();
			urlStream.close();
			curl.setMd5(Utils.getHashString(downloadFile));
			/* now add out document */
			if (ds!=null) {
				curSuccessNo=ds.idx.addDocToIndex(downloadFile, iw, dsi, false, curl);
				switch (curSuccessNo) {
					case 0: // good
						numSuccesses++;
						break;
					case 1: // bad
						numFailed++;
						break;
					case 2: // meta robots - no index
						numNoRobots++;
						break;
					}
				}
			numSpidered++;
			completedSpider=true;
			}
		catch (Exception eD) {
			errStr=eD.toString();
			setStatus(ERROR+" : "+errStr);
			eD.printStackTrace();
			isDead=true;
			}
		finally {
			curl.setSpidered(completedSpider);
			curl.setDead(isDead);
			setStatus(DOWNLOAD_COMPLETE+" "+shortUrl);
			}	
	} // end for iterating over links
	saveAllLinks();
	System.out.println(numSpidered+" total web pages spidered for links.");
	showMessage(SPIDERING_COMPLETE+" ("+Utils.concatStrToEnd(pageName,28)+") ", 
		numSpidered+" "+DOCS_IDXD+" "+getNumLinksFound()+" "+LINKS_FOUND+
		"\n\n"+ numSuccesses +" "+SUCCESSFUL_DOWNLOADS+
		"\n\n"+ numFailed +" "+FAILED_DOWNLOADS+
		"\n\n"+ numNoRobots +" "+META_DOWNLOADS);
} // end for getAllLinks

public boolean hasBaseUrl(String toCheck) {
	boolean retB=false;
	if (baseUrlFolder.equals("")) {
		retB=true;
		System.out.println("NO BASE URL!");
		}
	else {
		String lowerCheck=toCheck.toLowerCase();
		String lowerBase=baseUrlFolder.toLowerCase();
		if (lowerCheck.startsWith(lowerBase)) retB=true;
		}
	return retB;
}


public void setStatus(String toSet) { 
	statusString=toSet;
	if (ds!=null) ds.setStatus(toSet);
	}

public String getStatus() { return statusString; }	

public void doPossibleAdd(String page, String link) {
	link=link.trim();
	if ((!link.equals("")) && (link.indexOf("mailto:")==-1)) {
		if (link.indexOf("#")!=-1) {
			int anchorPos=link.indexOf("#");
			link=link.substring(0,anchorPos);
			}
	if (link.indexOf("\\")!=-1) link=Utils.replaceAll("\\", link, "/");
	if ((link.startsWith("'")) && (link.endsWith("'"))) link=link.substring(1, link.length()-1);
	// DETERMINE LINK SIZE
	long size=0;
	LinkValue lv=new LinkValue(page, link);
	String realUrl=lv.getRealLink();
	//
	if ((realUrl.toLowerCase().startsWith(domainUrl.toLowerCase())) 
		&& (!hasBogusDirs(realUrl))
		&& (hasBaseUrl(realUrl))
		&& (!skippType(realUrl))) {
		if (!realUrl.equals("")) size=Utils.getUrlSize(realUrl);
		if (size>0) {
			//
			 addLink(realUrl);
			}
		else {
			//setStatus(SKIPPING_LINK+" : "+realUrl+"\nSIZE="+size);
			skippedLinks.add(realUrl);
			numSkips++;
			}
		} // end for url has domain prefix
	else System.out.println("REAL URL "+realUrl+" Skipped becuase it doesn't start with "+domainUrl+", OR URL is not an indexible file type.");
	link="";
	}
} // end for doPossible add


public void addLink(String newUrl) {
	int curLen=0;
	int newLen=newUrl.length();
	String curUrlString="";
	Iterator it=links.iterator();
	int totLinks=links.size();
	int curPos=0;
	SpiderUrl turl;
	boolean okToAdd=true;
	if (!links.isEmpty()) {
		while(it.hasNext()) {
			turl=(SpiderUrl)it.next();
			curUrlString=turl.getUrl();
			curLen=curUrlString.length();
			if (newLen==curLen) {
				if (newUrl.equals(curUrlString)) {
					okToAdd=false;
					}
				else if (newUrl.toLowerCase().equals(curUrlString.toLowerCase())) {
					okToAdd=false;
					}
				}
			else if (newLen>curLen) {
				//curPos++;
				break;
				}
			curPos++;
			}
		}
	if (okToAdd) {
		SpiderUrl surl=new SpiderUrl(newUrl);
		links.add(curPos, surl);
		}

} // end for adding a link

public int getNumLinksFound() { return links.size(); }

public void saveAllLinks() {
	try {
		// now save the output to a file
		if (ds!=null) {
			outFile=Utils.addFolder(ds.indexDir, Utils.replaceAll(" ", dsi.getDesc(), "_")+".txt");
			}
		File saveFi=new File(outFile);
		FileWriter fw=new FileWriter(saveFi);
		PrintWriter pw=new PrintWriter(fw);
		SpiderUrl spy;
		Iterator it=links.iterator();
			while (it.hasNext()) {
			spy=(SpiderUrl)it.next();
			if (!spy.isDeadLink()) pw.println(spy.getUrl()+"|"+spy.getLastModified()+"|"+spy.getSize()+"|"+spy.getContentType()+"|"+spy.getMd5());
			}
		setStatus(links.size()+" total links found, "+numSkips+" links skipped.");
		if (skippedLinks.size()>0) {
			Iterator sit=skippedLinks.iterator();
			while (sit.hasNext()) {
				System.out.println((String)sit.next());
				}
			}
		fw.close();
		pw.close();
		}
	catch (Exception eR) {
		eR.printStackTrace();
		}
} // end for saveAllLinks

public void showMessage(String title, String details) {
	if (ds!=null) ds.showMessage(title, details);
	else System.out.println(title+"\n"+details);
}

public void checkFileForLinks(String origFile, String pageName) {
	String urlStr=pageName; String shortUrl=Utils.concatEnd(pageName, 33);
	if (ds!=null) setStatus(ds.dsLkngFoLnx+" "+shortUrl);
	String appletParam="";
	String dmUrl=Utils.getDomainUrl(urlStr);
	domainUrl=dmUrl;  String realTag=""; String lowerTag="";
	boolean inTag=false; String link="";
	long curPcnt=0; long lastPcnt=0;
	long thisFileSize=0;
	File testFi=new File(origFile);
	thisFileSize=testFi.length();
	long curDnldnd=0;
	StringBuffer tagBuf=new StringBuffer();
	baseUrlFolder=Utils.getBaseUrlFolder(urlStr);
	try {
		FileInputStream fi = new FileInputStream(origFile);
        	InputStreamReader isr = new InputStreamReader(fi);
		Reader in = new BufferedReader(isr);
		/* close the readers, etc...*/
		int ch;  char c=' ';
		while ((ch = in.read()) > -1) {
			c = (char)ch;
			curDnldnd++;
			if (thisFileSize>0) {
				if (curDnldnd%10==0) {
					curPcnt=(curDnldnd*100)/thisFileSize;
					if (curPcnt!=lastPcnt)  {
						lastPcnt=curPcnt;
						setStatus(ds.dsLkngFoLnx+" "+shortUrl+" --> ("+curPcnt+" %)");
						}
					}
				}
			if (c=='<') {
				inTag=true;
				tagBuf=new StringBuffer();
				} // end for start of a tag	
			else if (c=='>') {
				tagBuf.append(c);
				realTag=tagBuf.toString();
				lowerTag=realTag.toLowerCase();
				if (lowerTag.startsWith("</a")) {
					doPossibleAdd(pageName, link);
					}
				else if (lowerTag.startsWith("<area")) {
					link=Utils.getTagString("href=", realTag);
					link=Utils.getNormalUrl(link);
					if (restrictToSite) {
						doPossibleAdd(pageName, link);
						}
					else doPossibleAdd(pageName, link);
					}
				else if (lowerTag.startsWith("<param")) {
					appletParam=Utils.getTagString("name=", realTag);
					if (appletParam.toLowerCase().equals("url")) {
						link=Utils.getTagString("value=", realTag);
						link=Utils.getNormalUrl(link);
						if (restrictToSite) {
							doPossibleAdd(pageName, link);
							}
						else doPossibleAdd(pageName, link);
						}
					}
				else if (lowerTag.startsWith("<a href=")) {
					link=Utils.getTagString("href=", realTag);
					/* remove funky formatting */
					link=Utils.getNormalUrl(link);
					}
				inTag=false;
				} // end for end of a tag
			//
			if (inTag) tagBuf.append(c);
			} // end while reading
		in.close();
		fi.close();
		}
	catch (Exception eN) {
		eN.printStackTrace();
		}
} // end for checkFileForLinks

public int getNumDeletes() { return numDeletes; }
public int getNumUpdates() { return numChanges; }
public int getNumNew() { return numNew; }
public int getNumFails() { return numFails; }
public int getNumUnchanged() { return numUnChanged; }
public int getNumMetaNoIdx() { return numMetaNoIdx; }

public void update() throws IOException {
	numDeletes=0; numChanges=0; numNew=0;  numFails=0;
	numUnChanged=0;  numMetaNoIdx=0;
	ir = IndexReader.open(dsi.indexerPath);
	int maxNumDocs=ir.maxDoc();
	int maxTotal=maxNumDocs+maxNumDocs/10;
	ds.setStatus(ds.dsTtlDxInIdx+" "+maxNumDocs);
	int lastFound=0; String curFi=""; String dnldTmpName=""; String downloadFile="";
	SpiderUrl spy=null; String shortUrl="";
	Iterator it=links.iterator();
	/* assign index location to urls currently in the index */
	while (it.hasNext()) {
		spy=(SpiderUrl)it.next();
		curFi=spy.getUrl();
		shortUrl=Utils.concatEnd(curFi, 33);
		lastFound=ds.idx.spiderIndexNum(lastFound, curFi, ir);
		spy.setIndexLocation(lastFound);
		if (lastFound==-1) {
			System.out.println(curFi+" currently is not in the index");
			} // end for doc is not in the index
		} // end for iteration 
	/* now iterate over all the spider urls */
	int curSpiderNum=getNextUrlNo();
	SpiderUrl curSpider=null; int curIdxNum=0;
	int curSpiderStatus=0; int curAddedSuccess=-1;
	int curNumLinksFound=0; long curUrlSize=0;
	long curModified=0; String curHash="";
	int totalSpidered=0;
	while (curSpiderNum!=-1) {
		curSpider=getSpiderUrl(curSpiderNum);
		curSpiderStatus=0;
		curFi=curSpider.getUrl();
		curNumLinksFound=getNumLinksFound();
		curIdxNum=curSpider.getIndexLocation();
		curUrlSize=Utils.getUrlSize(curSpider.getUrl());
		curModified=Utils.getUrlModifiedDate(curSpider.getUrl());
		shortUrl=Utils.concatEnd(curFi, 33);
		dnldTmpName=ds.getDownloadFileName(spy.getContentType(), curFi.toLowerCase());
		downloadFile=Utils.addFolder(downloadFileDir, dnldTmpName);
		if (curUrlSize>maxFileSizeToGet) {
				String ex=SKIPPING_LINK_TOO_LONG+" ("+curUrlSize+" > "+maxFileSizeToGet+") "+shortUrl;
			        setStatus(ex);
				}
		else if (curIdxNum!=-1) {
			System.out.println(shortUrl+" IS IN INDEX!");
			ds.setStatus(ds.dsCkgFoUpdtsToDoc+" "+shortUrl+" ("+totalSpidered+" / "+curNumLinksFound+")");
			curSpiderStatus=Utils.urlStatus(curSpider, downloadFile);
			switch (curSpiderStatus) {
				case -1 : // broken url
					curSpider.setDead(true);
					ds.setStatus(ds.dsBknLink+" "+shortUrl);
					/* remove from index */
					ir.delete(curIdxNum);
					numDeletes++;
					break;
				case 0 : // same
					ds.setStatus(ds.lnkNoChanges+" "+shortUrl);
					numUnChanged++;
					totalSpidered++;
					break;
				case 1 : // changed
					ds.setStatus(ds.dsReIdxgLnk+" "+shortUrl);
					ir.delete(curIdxNum);
					ir.close();
					curHash=Utils.getHashString(downloadFile);
					curUrlSize=Utils.getFileSize(downloadFile);
					curSpider.setMd5(curHash);
					curSpider.setLastModified(curModified);
					curSpider.setSize(curUrlSize);
					iw = new IndexWriter(dsi.indexerPath, new StandardAnalyzer(), false);
					//iw.setUseCompoundFile(true);
					curAddedSuccess=ds.idx.addDocToIndex(downloadFile, iw, dsi, false, curSpider);
					iw.close();
					ir = IndexReader.open(dsi.indexerPath);
					if (curAddedSuccess==0) {
						numChanges++;
						totalSpidered++;
						}
					else if (curAddedSuccess==2) numMetaNoIdx++;
					else if (curAddedSuccess==1) {
						numFails++;
						System.out.println("INDEXING FAILED! "+shortUrl);
						}
					/* get links from downloaded file */
					if (isHtml(curSpider.getUrl().toLowerCase())) checkFileForLinks(downloadFile, curSpider.getUrl());
					break;
				} // end for switch
			} // end for its already in the index
		else {
			System.out.println(shortUrl+" IS IN INDEX!");
			/* get links from downloaded file */
			ds.setStatus(ds.dsSpiderNewUrl+" "+shortUrl+" ("+totalSpidered+" / "+curNumLinksFound+")");
			boolean downloadOk=Utils.downloadUrlToFile(curSpider.getUrl(), downloadFile);
			if (downloadOk) {
				curHash=Utils.getHashString(downloadFile);
				curSpider.setMd5(curHash);
				curSpider.setLastModified(curModified);
				curSpider.setSize(curUrlSize);
				iw = new IndexWriter(dsi.indexerPath, new StandardAnalyzer(), false);
				//iw.setUseCompoundFile(true);
				curAddedSuccess=ds.idx.addDocToIndex(downloadFile, iw, dsi, false, curSpider);
				iw.close();
				ir.close();
				ir = IndexReader.open(dsi.indexerPath);
				if (curAddedSuccess==0) {
					numNew++;
					totalSpidered++;
					}
				else if (curAddedSuccess==2) numMetaNoIdx++;
				else if (curAddedSuccess==1) {
					numFails++;
					System.out.println("INDEXING FAILED! "+shortUrl);
					}
				if (isHtml(curSpider.getUrl().toLowerCase())) checkFileForLinks(downloadFile, curSpider.getUrl());
				}
			else {
				setStatus(ds.dsBknLink+" "+shortUrl);
				curSpider.setDead(true);
				} // end for unable to download
			} // its not in the index
		/* last things to do */
		curSpider.setSpidered(true);
		curSpider.setLastModified(curModified);
		curSpider.setSize(curUrlSize);
		curSpiderNum=getNextUrlNo();
		if (curSpiderNum==-1) break;
		if (totalSpidered>maxTotal) break;
		} // end while iterating over the links
	ds.setStatus(ds.dsSpdrUpdteComp+" "+dsi.getDesc());
	saveAllLinks();
	/* update the date of the index */
	dsi.lastIndexed = DateTimeUtils.getToday();
	ir.close();
} // end for update

} // end for class clinkFinder
