import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.io.File;
import java.io.FileWriter;
import org.pdfbox.cos.COSDocument;
import org.pdfbox.cos.COSObject;
import org.pdfbox.encryption.DecryptDocument;
import org.pdfbox.exceptions.InvalidPasswordException;
import org.pdfbox.pdfparser.PDFParser;
import org.pdfbox.pdmodel.PDDocument;
import org.pdfbox.pdmodel.PDDocumentInformation;
import org.pdfbox.util.PDFTextStripper;
import multivalent.std.adaptor.pdf.*;
import tool.pdf.*;
import tool.doc.ExtractText;

public class PdfToText {
    public static final String DEFAULT_ENCODING = "ISO-8859-1";
    //"US-ASCII";
    //"UTF-8";
    private static final String PASSWORD = "-password";
    String fileName;
    String tempFile = "";
    String author = "";
    String title= "";
    String keywords = "";
    String summary = "";
    boolean returnBool=true;
    boolean gotText=false;
    boolean debug=true;

PdfToText(String fileName, String tempFile) {
        this.fileName = fileName;
        this.tempFile = tempFile;
    }
    
public void reset() {
  author="";
  title="";
  keywords="";
  summary = "";
  returnBool=false;
  gotText=false;
}
    
public boolean didParse() {
	try {
		parse();
		}
	catch (Exception eR) {
	        System.out.println(eR.toString());
		}
	finally {
		return  gotText;
		}
}
    
public void parse() throws Exception {
	returnBool=false;
	gotText=false;
        int currentArgumentIndex = 0;
        String password = "";
        String encoding = DEFAULT_ENCODING;
        PDFTextStripper stripper = new PDFTextStripper();
        InputStream input = null;
        Writer output = null;
        COSDocument document = null;
        input = new FileInputStream(fileName);
        document = parseDocument(input);
	  // encryption
        if (document.isEncrypted())  {
                DecryptDocument decryptor = new DecryptDocument(document);
                decryptor.decryptDocument(password);
                }
         // WRITE THE TEXT TO THE FILE
	 boolean finishedTextExtraction=false;
	   try {
            output = new OutputStreamWriter(new FileOutputStream(tempFile), encoding);
            stripper.writeText(document, output);
	    finishedTextExtraction=true;
		gotText=true;
		if (debug) System.out.println("Successfully stripped out text from ("+fileName+") \n\tto: ("+tempFile+")");
		 }
		catch (IOException e) {
			String errS=e.toString();
			System.out.println("PDF Box PARSE Error: "+errS);
			if (errS.toLowerCase().indexOf("font")!=-1) gotText=true;
			}
		// GET THE META DATA
	    try {
	      PDDocumentInformation info =(new PDDocument(document)).getDocumentInformation();
            author = info.getAuthor();
	    	title=info.getTitle();
	    	keywords=info.getKeywords();
	    	summary = info.getSubject();
		if ((summary == null) || (keywords==null) || (title==null) || (author == null)) returnBool=false;
		else returnBool=true;
		}
	    catch (Exception eR) {
	    	System.out.println("PDF ERROR OBTAINING META DATA:"+eR.toString());
	    	}

    }

public static void main(String[] args)   {
    int numArgs=args.length;
    if (numArgs<2) System.out.println("USAGE:\n\tjava PdfToText pdf_file.pdf tempTextFile.txt");
    else {
    	PdfToText ptt= new PdfToText(args[0], args[1]);
	try {
		ptt.doMultiValent();
		//ptt.parse();
		if ((ptt.success()) && (ptt.foundText())) {
			System.out.println("SUCCESS: \n\tTitle:"+ptt.getTitle()+"\n\tAuthor:"+ptt.getAuthor()+"\n\tSum:"+ptt.getSummary());
			}
		else if (ptt.foundText()) System.out.println("UNSUCCESSFUL FOR META DATA ONLY");
		else System.out.println("COMPLETELY UNSUCCESSFUL!");
		}
	catch (Exception eO) {
		System.out.println(eO.toString());
		}
    	}
    }

public boolean success() {
	if (debug) {
		if (!returnBool) System.out.println("Unable to get META DATA from PDF document: "+fileName);
		}
	return returnBool;
}

public boolean foundText() {
	if (debug) {
		if (!gotText) System.out.println("Unable to strip text from PDF document: "+fileName);
		}
	return gotText;
}
    
public String getTitle() {
	return title;
}

public String getAuthor() {
	return author;
}

public String getSummary() {
	return summary;
}

public String getKeywords() {
	return keywords;
}

public boolean doMultiValent() {
	boolean retB=false;
	File filein=new File(fileName);
	/*
	File fileout=new File(tempFile); 
	tool.pdf.Extract tpe=new tool.pdf.Extract();
	*/
	tool.doc.ExtractText tpe=new tool.doc.ExtractText();
	try {
		//tpe.setCache(new File(System.getProperty("java.io.tmpdir")));
		tpe.setVerbose(true);
		StringBuffer sb=new StringBuffer();
		sb.append(tpe.extract(filein.toURI(), "text/pdf"));
		saveFile(tempFile, sb);
		gotText=true;
		retB=true;
		}
	catch (Exception eR) {
	        System.out.println("Miltivalent text extraction failed:"+eR.toString());
		retB=false;
		}
	finally {
		return retB;
		}
}

public void saveFile(String fName, StringBuffer content) {
        String saveStr = "";
        boolean error = false;
        try {
            File saveFile = new File(fName);
            FileWriter writer = new FileWriter(saveFile);
            saveStr = content.toString();
            int saveStrLen = saveStr.length();
            for (int i = 0; i < saveStrLen; i++) writer.write((int)saveStr.charAt(i));
            writer.close();
        }
        catch (Exception eF) {
            System.out.println("Error saving stringbuffer " + eF.toString() + " " + fName);
            error = true;
        }

        if (!error) {
            System.out.println(fileName +" was saved");
        }
    }

private static COSDocument parseDocument(InputStream input) throws IOException {
        PDFParser parser = new PDFParser(input);
        parser.parse();
        return parser.getDocument();
    }
    
}
